@@ -6,6 +6,7 @@
 #' @param length A non-negative integer specifying the desired length.
 #' @param type A character string specifying the type of gpuVector.  Default
 #' is NULL where type is inherited from the source data type.
+#' @param ctx_id An integer specifying the object's context
 #' @param ... Additional method to pass to gpuVector methods
 #' @return A gpuVector object
 #' @docType methods
@@ -20,14 +21,18 @@ setGeneric("gpuVector", function(data, length, type=NULL, ...){
 #' @aliases gpuVector,vector
 setMethod('gpuVector', 
           signature(data = 'vector', length = 'missing'),
-          function(data, type=NULL){
+          function(data, type=NULL, ctx_id = NULL){
               
-              if (is.null(type)) type <- typeof(data)
+              if (is.null(type)) {
+                  type <- switch(typeof(data),
+                                 "integer" = "integer",
+                                 getOption("gpuR.default.type"))
+              }
               
               device <- currentDevice()
               
-              context_index <- currentContext()
-              device_index <- device$device_index
+              context_index <- ifelse(is.null(ctx_id), currentContext(), as.integer(ctx_id))
+              device_index <- as.integer(device$device_index)
               device_type <- device$device_type
               device_name <- switch(device_type,
                                     "gpu" = gpuInfo(device_idx = as.integer(device_index))$deviceName,
@@ -37,12 +42,7 @@ setMethod('gpuVector',
               platform_index <- currentPlatform()$platform_index
               platform_name <- platformInfo(platform_index)$platformName
               
-              if(type == "double" & !deviceHasDouble(platform_index, device_index)){
-                  stop("Double precision not supported for current device. 
-                       Try setting 'type = 'float'' or change device if multiple available.")
-              }
-              
-              data = switch(type,
+              out = switch(type,
                             integer = {
                                 new("igpuVector", 
                                     address=sexpVecToEigenVecXptr(data, 
@@ -66,6 +66,7 @@ setMethod('gpuVector',
                                     .device = device_name)
                             },
                             double = {
+                                assert_has_double(platform_index, device_index)
                                 new("dgpuVector",
                                     address = sexpVecToEigenVecXptr(data,
                                                                     length(data),
@@ -80,16 +81,17 @@ setMethod('gpuVector',
                                  or unimplemented data type")
               )
               
-              return(data)
+              return(out)
           },
-          valueClass = "gpuVector")
+          valueClass = "gpuVector"
+)
 
 
 #' @rdname gpuVector-methods
 #' @aliases gpuVector,missingOrNULL
 setMethod('gpuVector', 
           signature(data = 'missingOrNULL'),
-          function(data, length, type=NULL){
+          function(data, length, type=NULL, ctx_id = NULL){
               
               if (is.null(type)) type <- getOption("gpuR.default.type")
               if (length <= 0) stop("length must be a positive integer")
@@ -97,8 +99,8 @@ setMethod('gpuVector',
               
               device <- currentDevice()
               
-              context_index <- currentContext()
-              device_index <- device$device_index
+              context_index <- ifelse(is.null(ctx_id), currentContext(), as.integer(ctx_id))
+              device_index <- as.integer(device$device_index)
               device_type <- device$device_type
               device_name <- switch(device_type,
                                     "gpu" = gpuInfo(device_idx = as.integer(device_index))$deviceName,
@@ -108,12 +110,7 @@ setMethod('gpuVector',
               platform_index <- currentPlatform()$platform_index
               platform_name <- platformInfo(platform_index)$platformName
               
-              if(type == "double" & !deviceHasDouble(platform_index, device_index)){
-                  stop("Double precision not supported for current device. 
-                       Try setting 'type = 'float'' or change device if multiple available.")
-              }
-              
-              data = switch(type,
+              out = switch(type,
                             integer = {
                                 new("igpuVector", 
                                     address=emptyEigenVecXptr(length, 4L),
@@ -133,6 +130,7 @@ setMethod('gpuVector',
                                     .device = device_name)
                             },
                             double = {
+                                assert_has_double(platform_index, device_index)
                                 new("dgpuVector",
                                     address = emptyEigenVecXptr(length, 8L),
                                     .context_index = context_index,
@@ -145,6 +143,7 @@ setMethod('gpuVector',
                                  or unimplemented data type")
               )
               
-              return(data)
+              return(out)
           },
-          valueClass = "gpuVector")
+          valueClass = "gpuVector"
+)
