@@ -1,4 +1,51 @@
 
+#' @export
+as.vector.vclVector <- function(x, mode = "any"){
+    out <- x[]
+    return(out)
+}
+
+
+#' @rdname vclVector-methods
+#' @param shared Logical indicating if memory should be shared with \code{x}
+#' @export
+as.vclVector <- function (data, shared, ...) {
+    UseMethod("as.vclVector", data)
+}
+
+#' @export
+as.vclVector.vclMatrix <- function(data, shared = TRUE, ...){
+    if(shared){
+        
+        switch(typeof(data),
+               "integer" = return(new("ivclVector", 
+                                      address=vclMatTovclVec(data@address, 4L),
+                                      .context_index = data@.context_index,
+                                      .platform_index = data@.platform_index,
+                                      .platform = data@.platform,
+                                      .device_index = data@.device_index,
+                                      .device = data@.device)),
+               "float" = return(new("fvclVector", 
+                                    address=vclMatTovclVec(data@address, 6L),
+                                    .context_index = data@.context_index,
+                                    .platform_index = data@.platform_index,
+                                    .platform = data@.platform,
+                                    .device_index = data@.device_index,
+                                    .device = data@.device)),
+               "double" = return(new("dvclVector", 
+                                     address=vclMatTovclVec(data@address, 8L),
+                                     .context_index = data@.context_index,
+                                     .platform_index = data@.platform_index,
+                                     .platform = data@.platform,
+                                     .device_index = data@.device_index,
+                                     .device = data@.device))
+        )
+    }else{
+        stop("copy constructor not implemented yet")
+    }
+}
+
+
 #' @rdname extract-methods
 #' @export
 setMethod("[",
@@ -32,7 +79,7 @@ setMethod("[",
 setMethod("[<-",
           signature(x = "vclVector", i = "numeric", j = "missing", value="numeric"),
           function(x, i, j, value) {
-              if(length(value) != 1){
+              if(length(value) > 1){
                   stop("number of items to replace is not a multiple of replacement length")
               }
               
@@ -43,6 +90,8 @@ setMethod("[<-",
                      "double" = vclVecSetElement(x@address, i, value, 8L),
                      stop("type not recognized")
               )
+              
+              
               return(x)
           })
 
@@ -64,6 +113,43 @@ setMethod("[<-",
               return(x)
           })
 
+#' @rdname extract-methods
+#' @export
+setMethod("[<-",
+          signature(x = "vclVector", i = "missing", j = "missing", value="numeric"),
+          function(x, i, j, value) {
+              if(length(value) > length(x)){
+                  stop("number of items to replace is not a multiple of replacement length")
+              }
+              
+              switch(typeof(x),
+                     "integer" = vclSetVector(x@address, value, 4L, x@.context_index - 1),
+                     "float" = vclSetVector(x@address, value, 6L, x@.context_index - 1),
+                     "double" = vclSetVector(x@address, value, 8L, x@.context_index - 1),
+                     stop("unsupported matrix type")
+              )
+              
+              return(x)
+          })
+
+
+#' @rdname extract-methods
+#' @export
+setMethod("[<-",
+          signature(x = "vclVector", i = "missing", j = "missing", value = "vclVector"),
+          function(x, i, j, value) {
+              
+              switch(typeof(x),
+                     "integer" = vclSetVCLVector(x@address, value@address, 4L),
+                     "float" = vclSetVCLVector(x@address, value@address, 6L),
+                     "double" = vclSetVCLVector(x@address, value@address, 8L),
+                     stop("unsupported matrix type")
+              )
+              
+              return(x)
+          })
+
+
 #' @rdname grapes-times-grapes-methods
 #' @export
 setMethod("%*%", signature(x="vclVector", y = "vclVector"),
@@ -122,20 +208,20 @@ setMethod("Arith", c(e1="numeric", e2="vclVector"),
               op = .Generic[[1]]
               switch(op,
                      `+` = {
-                         e1 = vclVector(rep(e1, length(e2)), type=typeof(e2))
+                         e1 = vclVector(rep(e1, length(e2)), type=typeof(e2), ctx_id = e2@.context_index)
                          vclVec_axpy(1, e1, e2)
                      },
                      `-` = {
-                         e1 = vclVector(rep(e1, length(e2)), type=typeof(e2))
+                         e1 = vclVector(rep(e1, length(e2)), type=typeof(e2), ctx_id = e2@.context_index)
                          vclVec_axpy(-1, e2, e1)
                      },
                      `*` = vclVecScalarMult(e2, e1),
                      `/` = {
-                         e1 = vclVector(rep(e1, length(e2)), type=typeof(e2))
+                         e1 = vclVector(rep(e1, length(e2)), type=typeof(e2), ctx_id = e2@.context_index)
                          vclVecElemDiv(e1, e2)
                      },
                      `^` = {
-                         e1 <- vclVector(rep(e1, length(e2)), type=typeof(e2))
+                         e1 <- vclVector(rep(e1, length(e2)), type=typeof(e2), ctx_id = e2@.context_index)
                          vclVecElemPow(e1, e2)
                      },
                      stop("undefined operation")
@@ -154,11 +240,11 @@ setMethod("Arith", c(e1="vclVector", e2="numeric"),
               op = .Generic[[1]]
               switch(op,
                      `+` = {
-                         e2 = vclVector(rep(e2, length(e1)), type=typeof(e1))
+                         e2 = vclVector(rep(e2, length(e1)), type=typeof(e1), ctx_id = e1@.context_index)
                          vclVec_axpy(1, e1, e2)
                      },
                      `-` = {
-                         e2 = vclVector(rep(e2, length(e1)), type=typeof(e1))
+                         e2 = vclVector(rep(e2, length(e1)), type=typeof(e1), ctx_id = e1@.context_index)
                          vclVec_axpy(-1, e2, e1)
                      },
                      `*` = vclVecScalarMult(e1, e2),
@@ -246,9 +332,9 @@ setMethod("Summary", c(x="vclVector"),
 setMethod('length', signature(x = "vclVector"),
           function(x) {
               switch(typeof(x),
-                     "integer" = return(vcl_igpuVec_size(x@address)),
-                     "float" = return(vcl_fgpuVec_size(x@address)),
-                     "double" = return(vcl_dgpuVec_size(x@address))
+                     "integer" = return(cpp_vclVector_size(x@address, 4L)),
+                     "float" = return(cpp_vclVector_size(x@address, 6L)),
+                     "double" = return(cpp_vclVector_size(x@address, 8L))
               )
               
           }
@@ -258,15 +344,38 @@ setMethod('length', signature(x = "vclVector"),
 setMethod("deepcopy", signature(object ="vclVector"),
           function(object){
               
-              out <- switch(typeof(object),
-                            "integer" = new("ivclVector",
-                                            address = cpp_deepcopy_vclVector(object@address, 4L)),
-                            "float" = new("fvclVector", 
-                                          address = cpp_deepcopy_vclVector(object@address, 6L)),
-                            "double" = new("dvclVector", 
-                                           address = cpp_deepcopy_vclVector(object@address, 8L)),
-                            stop("unrecognized type")
-              )
+              out <- 
+                  switch(typeof(object),
+                         "integer" = new("ivclVector",
+                                         address = cpp_deepcopy_vclVector(
+                                             object@address, 
+                                             4L, 
+                                             object@.context_index - 1),
+                                         .context_index = object@.context_index,
+                                         .platform_index = object@.platform_index,
+                                         .platform = object@.platform,
+                                         .device_index = object@.device_index,
+                                         .device = object@.device),
+                         "float" = new("fvclVector", 
+                                       address = cpp_deepcopy_vclVector(
+                                           object@address, 6L, 
+                                           object@.context_index - 1),
+                                       .context_index = object@.context_index,
+                                       .platform_index = object@.platform_index,
+                                       .platform = object@.platform,
+                                       .device_index = object@.device_index,
+                                       .device = object@.device),
+                         "double" = new("dvclVector", 
+                                        address = cpp_deepcopy_vclVector(
+                                            object@address, 8L, 
+                                            object@.context_index - 1),
+                                        .context_index = object@.context_index,
+                                        .platform_index = object@.platform_index,
+                                        .platform = object@.platform,
+                                        .device_index = object@.device_index,
+                                        .device = object@.device),
+                         stop("unrecognized type")
+                  )
               return(out)
           })
 
@@ -281,11 +390,23 @@ setMethod("slice",
               ptr <- switch(typeof(object),
                             "float" = {
                                 address <- cpp_vclVector_slice(object@address, start, end, 6L)
-                                new("fvclVectorSlice", address = address)
+                                new("fvclVectorSlice", 
+                                    address = address,
+                                    .context_index = object@.context_index,
+                                    .platform_index = object@.platform_index,
+                                    .platform = object@.platform,
+                                    .device_index = object@.device_index,
+                                    .device = object@.device)
                             },
                             "double" = {
                                 address <- cpp_vclVector_slice(object@address, start, end, 8L)
-                                new("dvclVectorSlice", address = address)
+                                new("dvclVectorSlice", 
+                                    address = address,
+                                    .context_index = object@.context_index,
+                                    .platform_index = object@.platform_index,
+                                    .platform = object@.platform,
+                                    .device_index = object@.device_index,
+                                    .device = object@.device)
                             },
                             stop("type not recognized")
               )
