@@ -1,9 +1,6 @@
 library(gpuR)
 context("CPU gpuVector math operations")
 
-# set option to use CPU instead of GPU
-options(gpuR.default.device = "cpu")
-
 # set seed
 set.seed(123)
 
@@ -17,14 +14,14 @@ A <- rnorm(ORDER)
 B <- rnorm(ORDER)
 
 
-test_that("gpuVector Single Precision Matrix Element-Wise Trignometry", {
+test_that("CPU gpuVector Single Precision Matrix Element-Wise Trignometry", {
     has_cpu_skip()
     
     Sin <- sin(A)
-    Asin <- asin(A)
+    Asin <- suppressWarnings(asin(A))
     Hsin <- sinh(A)
     Cos <- cos(A)
-    Acos <- acos(A)
+    Acos <- suppressWarnings(acos(A))
     Hcos <- cosh(A)
     Tan <- tan(A) 
     Atan <- atan(A)
@@ -63,14 +60,14 @@ test_that("gpuVector Single Precision Matrix Element-Wise Trignometry", {
                  info="hyperbolic tan float matrix elements not equivalent")  
 })
 
-test_that("gpuVector Double Precision Matrix Element-Wise Trignometry", {
+test_that("CPU gpuVector Double Precision Matrix Element-Wise Trignometry", {
     has_cpu_skip()
     
     Sin <- sin(A)
-    Asin <- asin(A)
+    Asin <- suppressWarnings(asin(A))
     Hsin <- sinh(A)
     Cos <- cos(A)
-    Acos <- acos(A)
+    Acos <- suppressWarnings(acos(A))
     Hcos <- cosh(A)
     Tan <- tan(A) 
     Atan <- atan(A)
@@ -109,53 +106,151 @@ test_that("gpuVector Double Precision Matrix Element-Wise Trignometry", {
                  info="hyperbolic tan double matrix elements not equivalent") 
 })
 
-# 
-# test_that("gpuVector Single Precision Matrix Element-Wise Logs", {
-#     
-#     R_log <- log(A)
-#     R_log10 <- log10(A)
-#     R_log2 <- log(A, base=2)
-#     
-#     fgpuA <- gpuVector(A, type="float")
-#     
-#     fgpu_log <- log(fgpuA)
-#     fgpu_log10 <- log10(fgpuA)
-#     fgpu_log2 <- log(fgpuA, base=2)
-#     
-#     expect_is(fgpu_log, "fgpuVector")
-#     expect_is(fgpu_log10, "fgpuVector")
-#     expect_is(fgpu_log2, "fgpuVector")
-#     expect_equal(fgpu_log[,], R_log, tolerance=1e-07, 
-#                  info="log float matrix elements not equivalent")  
-#     expect_equal(fgpu_log10[,], R_log10, tolerance=1e-07, 
-#                  info="log10 float matrix elements not equivalent")  
-#     expect_equal(fgpu_log2[,], R_log2, tolerance=1e-07, 
-#                  info="base log float matrix elements not equivalent") 
-# })
-# 
-# test_that("gpuVector Double Precision Matrix Element-Wise Logs", {
-#     
-#     has_gpu_skip()
-#     
-#     R_log <- log(A)
-#     R_log10 <- log10(A)
-#     R_log2 <- log(A, base=2)
-#     
-#     fgpuA <- gpuVector(A, type="double")
-#     
-#     fgpu_log <- log(fgpuA)
-#     fgpu_log10 <- log10(fgpuA)
-#     fgpu_log2 <- log(fgpuA, base=2)
-#     
-#     expect_is(fgpu_log, "dgpuVector")
-#     expect_is(fgpu_log10, "dgpuVector")
-#     expect_is(fgpu_log2, "dgpuVector")
-#     expect_equal(fgpu_log[,], R_log, tolerance=.Machine$double.eps ^ 0.5, 
-#                  info="log float matrix elements not equivalent")  
-#     expect_equal(fgpu_log10[,], R_log10, tolerance=.Machine$double.eps ^ 0.5, 
-#                  info="log10 float matrix elements not equivalent")  
-#     expect_equal(fgpu_log2[,], R_log2, tolerance=.Machine$double.eps ^ 0.5, 
-#                  info="base log float matrix elements not equivalent") 
-# })
+test_that("CPU gpuVector Single Precision Element-Wise Logs", {
+    
+    has_cpu_skip()
+    
+    R_log <- suppressWarnings(log(A))
+    R_log10 <- suppressWarnings(log10(A))
+    R_log2 <- suppressWarnings(log(A, base=2))
+    
+    fgpuA <- gpuVector(A, type="float")
+    
+    fgpu_log <- log(fgpuA)
+    fgpu_log10 <- log10(fgpuA)
+    fgpu_log2 <- log(fgpuA, base=2)
+    
+    expect_is(fgpu_log, "fgpuVector")
+    expect_is(fgpu_log10, "fgpuVector")
+    expect_is(fgpu_log2, "fgpuVector")
+    expect_equal(fgpu_log[,], R_log, tolerance=1e-07, 
+                 info="log float matrix elements not equivalent")  
+    expect_equal(fgpu_log10[,], R_log10, tolerance=1e-07, 
+                 info="log10 float matrix elements not equivalent")  
+    expect_equal(fgpu_log2[,], R_log2, tolerance=1e-0, 
+                 info="base log float matrix elements not equivalent") 
+})
+
+test_that("CPU gpuVector Double Precision Element-Wise Logs", {
+    
+    has_cpu_skip()
+    
+    R_log <- suppressWarnings(log(A))
+    R_log10 <- suppressWarnings(log10(A))
+    R_log2 <- suppressWarnings(log(A, base=2))
+    
+    fgpuA <- gpuVector(A, type="double")
+    
+    fgpu_log <- log(fgpuA)
+    fgpu_log10 <- log10(fgpuA)
+    fgpu_log2 <- log(fgpuA, base=2)
+    
+    expect_is(fgpu_log, "dgpuVector")
+    expect_is(fgpu_log10, "dgpuVector")
+    expect_is(fgpu_log2, "dgpuVector")
+    expect_equal(fgpu_log[,], R_log, tolerance=.Machine$double.eps ^ 0.5, 
+                 info="log double matrix elements not equivalent")  
+    expect_equal(fgpu_log10[,], R_log10, tolerance=.Machine$double.eps ^ 0.5, 
+                 info="log10 double matrix elements not equivalent")  
+    expect_equal(fgpu_log2[,], R_log2, tolerance=.Machine$double.eps ^ 0.5, 
+                 info="base log double matrix elements not equivalent") 
+})
+
+test_that("CPU gpuVector Single Precision Exponential", {
+    
+    has_cpu_skip()
+    
+    R_exp <- exp(A)
+    
+    fgpuA <- gpuVector(A, type="float")
+    
+    fgpu_exp <- exp(fgpuA)
+    
+    expect_is(fgpu_exp, "fgpuVector")
+    expect_equal(fgpu_exp[,], R_exp, tolerance=1e-07, 
+                 info="exp float vector elements not equivalent")  
+})
+
+test_that("CPU gpuVector Double Precision Exponential", {
+    
+    has_cpu_skip()
+    
+    R_exp <- exp(A)
+    
+    fgpuA <- gpuVector(A, type="double")
+    
+    fgpu_exp <- exp(fgpuA)
+    
+    expect_is(fgpu_exp, "dgpuVector")
+    expect_equal(fgpu_exp[,], R_exp, tolerance=.Machine$double.eps^0.5, 
+                 info="exp double vector elements not equivalent")  
+})
+
+test_that("CPU gpuVector Single Precision Absolute Value", {
+    
+    has_cpu_skip()
+    
+    R_abs <- abs(A)
+    
+    fgpuA <- gpuVector(A, type="float")
+    
+    fgpu_abs <- abs(fgpuA)
+    
+    expect_is(fgpu_abs, "fgpuVector")
+    expect_equal(fgpu_abs[,], R_abs, tolerance=1e-07, 
+                 info="abs float vector elements not equivalent")  
+})
+
+test_that("CPU gpuVector Double Precision Absolute Value", {
+    
+    has_cpu_skip()
+    
+    R_abs <- abs(A)
+    
+    fgpuA <- gpuVector(A, type="double")
+    
+    fgpu_abs <- abs(fgpuA)
+    
+    expect_is(fgpu_abs, "dgpuVector")
+    expect_equal(fgpu_abs[,], R_abs, tolerance=.Machine$double.eps^0.5, 
+                 info="abs double vector elements not equivalent")  
+})
+
+test_that("CPU gpuVector Single Precision Maximum/Minimum", {
+    
+    has_cpu_skip()
+    
+    R_max <- max(A)
+    R_min <- min(A)
+    
+    fgpuA <- gpuVector(A, type="float")
+    
+    fgpu_max <- max(fgpuA)
+    fgpu_min <- min(fgpuA)
+    
+    expect_is(fgpu_max, "numeric")
+    expect_equal(fgpu_max, R_max, tolerance=1e-07, 
+                 info="max float vector element not equivalent")  
+    expect_equal(fgpu_min, R_min, tolerance=1e-07, 
+                 info="min float vector element not equivalent")  
+})
+
+test_that("CPU gpuVector Double Precision Maximum/Minimum", {
+    
+    has_cpu_skip()
+    
+    R_max <- max(A)
+    R_min <- min(A)
+    
+    fgpuA <- gpuVector(A, type="double")
+    
+    fgpu_max <- max(fgpuA)
+    fgpu_min <- min(fgpuA)
+    
+    expect_is(fgpu_max, "numeric")
+    expect_equal(fgpu_max, R_max, tolerance=.Machine$double.eps^0.5, 
+                 info="max double vector element not equivalent") 
+    expect_equal(fgpu_min, R_min, tolerance=.Machine$double.eps^0.5, 
+                 info="min double vector element not equivalent")  
+})
 
-options(gpuR.default.device = "gpu")
