@@ -8,9 +8,6 @@ set.seed(123)
 # ignore warnings (logs and arc trigs)
 options(warn=-1)
 
-# set option to use CPU instead of GPU
-options(gpuR.default.device = "cpu")
-
 ORDER <- 4
 
 # Base R objects
@@ -18,14 +15,14 @@ A <- rnorm(ORDER)
 B <- rnorm(ORDER)
 
 
-test_that("vclVector Single Precision Matrix Element-Wise Trignometry", {
+test_that("CPU vclVector Single Precision Matrix Element-Wise Trignometry", {
     has_cpu_skip()
     
     Sin <- sin(A)
-    Asin <- asin(A)
+    Asin <- suppressWarnings(asin(A))
     Hsin <- sinh(A)
     Cos <- cos(A)
-    Acos <- acos(A)
+    Acos <- suppressWarnings(acos(A))
     Hcos <- cosh(A)
     Tan <- tan(A) 
     Atan <- atan(A)
@@ -64,14 +61,14 @@ test_that("vclVector Single Precision Matrix Element-Wise Trignometry", {
                  info="hyperbolic tan float matrix elements not equivalent")  
 })
 
-test_that("vclVector Double Precision Matrix Element-Wise Trignometry", {
+test_that("CPU vclVector Double Precision Matrix Element-Wise Trignometry", {
     has_cpu_skip()
 
     Sin <- sin(A)
-    Asin <- asin(A)
+    Asin <- suppressWarnings(asin(A))
     Hsin <- sinh(A)
     Cos <- cos(A)
-    Acos <- acos(A)
+    Acos <- suppressWarnings(acos(A))
     Hcos <- cosh(A)
     Tan <- tan(A) 
     Atan <- atan(A)
@@ -111,12 +108,12 @@ test_that("vclVector Double Precision Matrix Element-Wise Trignometry", {
 })
 
 
-test_that("vclVector Single Precision Matrix Element-Wise Logs", {
+test_that("CPU vclVector Single Precision Matrix Element-Wise Logs", {
     has_cpu_skip()
     
-    R_log <- log(A)
-    R_log10 <- log10(A)
-    R_log2 <- log(A, base=2)
+    R_log <- suppressWarnings(log(A))
+    R_log10 <- suppressWarnings(log10(A))
+    R_log2 <- suppressWarnings(log(A, base=2))
     
     fgpuA <- vclVector(A, type="float")
     
@@ -135,12 +132,12 @@ test_that("vclVector Single Precision Matrix Element-Wise Logs", {
                  info="base log float matrix elements not equivalent") 
 })
 
-test_that("vclVector Double Precision Matrix Element-Wise Logs", {
+test_that("CPU vclVector Double Precision Matrix Element-Wise Logs", {
     has_cpu_skip()
     
-    R_log <- log(A)
-    R_log10 <- log10(A)
-    R_log2 <- log(A, base=2)
+    R_log <- suppressWarnings(log(A))
+    R_log10 <- suppressWarnings(log10(A))
+    R_log2 <- suppressWarnings(log(A, base=2))
     
     fgpuA <- vclVector(A, type="double")
     
@@ -159,4 +156,101 @@ test_that("vclVector Double Precision Matrix Element-Wise Logs", {
                  info="log base 2 double matrix elements not equivalent") 
 })
 
-options(gpuR.default.device = "gpu")
+
+test_that("CPU vclVector Single Precision Exponential", {
+    
+    has_cpu_skip()
+    
+    R_exp <- exp(A)
+    
+    fvclA <- vclVector(A, type="float")
+    
+    fvcl_exp <- exp(fvclA)
+    
+    expect_is(fvcl_exp, "fvclVector")
+    expect_equal(fvcl_exp[,], R_exp, tolerance=1e-07, 
+                 info="exp float vector elements not equivalent")  
+})
+
+test_that("CPU vclVector Double Precision Exponential", {
+    
+    has_cpu_skip()
+    
+    R_exp <- exp(A)
+    
+    fvclA <- vclVector(A, type="double")
+    
+    fvcl_exp <- exp(fvclA)
+    
+    expect_is(fvcl_exp, "dvclVector")
+    expect_equal(fvcl_exp[,], R_exp, tolerance=.Machine$double.eps^0.5, 
+                 info="exp double vector elements not equivalent")  
+})
+
+test_that("CPU vclVector Single Precision Absolute Value", {
+    
+    has_cpu_skip()
+    
+    R_abs <- abs(A)
+    
+    fvclA <- vclVector(A, type="float")
+    
+    fvcl_abs <- abs(fvclA)
+    
+    expect_is(fvcl_abs, "fvclVector")
+    expect_equal(fvcl_abs[,], R_abs, tolerance=1e-07, 
+                 info="abs float vector elements not equivalent")  
+})
+
+test_that("CPU vclVector Double Precision Absolute Value", {
+    
+    has_cpu_skip()
+    
+    R_abs <- abs(A)
+    
+    fvclA <- vclVector(A, type="double")
+    
+    fvcl_abs <- abs(fvclA)
+    
+    expect_is(fvcl_abs, "dvclVector")
+    expect_equal(fvcl_abs[,], R_abs, tolerance=.Machine$double.eps^0.5, 
+                 info="abs double vector elements not equivalent")  
+})
+
+test_that("CPU vclVector Single Precision Maximum/Minimum", {
+    
+    has_cpu_skip()
+    
+    R_max <- max(A)
+    R_min <- min(A)
+    
+    fvclA <- vclVector(A, type="float")
+    
+    fvcl_max <- max(fvclA)
+    fvcl_min <- min(fvclA)
+    
+    expect_is(fvcl_max, "numeric")
+    expect_equal(fvcl_max, R_max, tolerance=1e-07, 
+                 info="max float vector element not equivalent")  
+    expect_equal(fvcl_min, R_min, tolerance=1e-07, 
+                 info="min float vector element not equivalent")  
+})
+
+test_that("CPU vclVector Double Precision Maximum/Minimum", {
+    
+    has_cpu_skip()
+    
+    R_max <- max(A)
+    R_min <- min(A)
+    
+    fvclA <- vclVector(A, type="double")
+    
+    fvcl_max <- max(fvclA)
+    fvcl_min <- min(fvclA)
+    
+    expect_is(fvcl_max, "numeric")
+    expect_equal(fvcl_max, R_max, tolerance=.Machine$double.eps^0.5, 
+                 info="max double vector element not equivalent") 
+    expect_equal(fvcl_min, R_min, tolerance=.Machine$double.eps^0.5, 
+                 info="min double vector element not equivalent")  
+})
