@@ -23,14 +23,15 @@ test_that("gpuMatrix Single Precision Matrix multiplication", {
     
     fgpuA <- gpuMatrix(A, type="float")
     fgpuB <- gpuMatrix(B, type="float")
-    
-#     print(fgpuA[])
+    fgpuE <- gpuMatrix(E, type = "float")
     
     fgpuC <- fgpuA %*% fgpuB
     
     expect_is(fgpuC, "fgpuMatrix")
     expect_equal(fgpuC[,], C, tolerance=1e-07, 
                  info="float matrix elements not equivalent")  
+    expect_error(fgpuA %*% fgpuE,
+                 info = "error not thrown for non-conformant matrices")
 })
 
 test_that("gpuMatrix Single Precision Matrix Subtraction", {
@@ -223,14 +224,18 @@ test_that("gpuMatrix Single Precision Scalar Matrix Power", {
     has_gpu_skip()
     
     C <- A^2
+    C2 <- 2^A
     
     dgpuA <- gpuMatrix(A, type="float")
     
     dgpuC <- dgpuA^2
+    dgpuC2 <- 2^dgpuA
     
     expect_is(dgpuC, "fgpuMatrix")
     expect_equal(dgpuC[,], C, tolerance=1e-07, 
                  info="float matrix elements not equivalent") 
+    expect_equal(dgpuC2[,], C2, tolerance=1e-07, 
+                 info="float matrix elements not equivalent") 
 })
 
 test_that("gpuMatrix Single Precision crossprod", {
@@ -302,34 +307,69 @@ test_that("gpuMatrix Single Precision transpose", {
 # Integer tests
 
 test_that("gpuMatrix Integer Matrix multiplication", {
-    
+
     has_gpu_skip()
-    
+
     Cint <- Aint %*% Bint
-    
+
     igpuA <- gpuMatrix(Aint, type="integer")
     igpuB <- gpuMatrix(Bint, type="integer")
-    
+
     igpuC <- igpuA %*% igpuB
-    
-    expect_equivalent(igpuC[,], Cint, 
-                      info="integer matrix elements not equivalent")      
+
+    expect_equivalent(igpuC[,], Cint,
+                      info="integer matrix elements not equivalent")
 })
 
 test_that("gpuMatrix Integer Matrix Subtraction", {
+
+    has_gpu_skip()
+
+    Cint <- Aint - Bint
+
+    igpuA <- gpuMatrix(Aint, type="integer")
+    igpuB <- gpuMatrix(Bint, type="integer")
+
+    igpuC <- igpuA - igpuB
+
+    expect_is(igpuC, "igpuMatrix")
+    expect_equal(igpuC[,], Cint,
+                 info="integer matrix elements not equivalent")
+})
+
+test_that("gpuMatrix Integer Precision Scalar Matrix Subtraction", {
     
     has_gpu_skip()
     
-    Cint <- Aint - Bint
+    C <- Aint - 1L
+    C2 <- 1L - Aint
     
-    igpuA <- gpuMatrix(Aint, type="integer")
-    igpuB <- gpuMatrix(Bint, type="integer")
+    fgpuA <- gpuMatrix(Aint, type="integer")
     
-    igpuC <- igpuA - igpuB
+    fgpuC <- fgpuA - 1L
+    fgpuC2 <- 1L - fgpuA
     
-    expect_is(igpuC, "igpuMatrix")
-    expect_equal(igpuC[,], Cint, 
-                 info="integer matrix elements not equivalent")  
+    expect_is(fgpuC, "igpuMatrix")
+    expect_equal(fgpuC[,], C, 
+                 info="integer matrix elements not equivalent") 
+    expect_is(fgpuC2, "igpuMatrix")
+    expect_equal(fgpuC2[,], C2,
+                 info="intger matrix elements not equivalent") 
+})
+
+test_that("gpuMatrix Integer Precision Unary Scalar Matrix Subtraction", {
+    
+    has_gpu_skip()
+    
+    C <- -Aint
+    
+    fgpuA <- gpuMatrix(Aint, type="integer")
+    
+    fgpuC <- -fgpuA
+    
+    expect_is(fgpuC, "igpuMatrix")
+    expect_equal(fgpuC[,], C,
+                 info="integer matrix elements not equivalent") 
 })
 
 test_that("gpuMatrix Integer Matrix Addition", {
@@ -345,9 +385,206 @@ test_that("gpuMatrix Integer Matrix Addition", {
     
     expect_is(igpuC, "igpuMatrix")
     expect_equal(igpuC[,], Cint,
+                 info="integer matrix elements not equivalent")
+})
+
+test_that("gpuMatrix Integer Precision Scalar Matrix Addition", {
+    
+    has_gpu_skip()
+    
+    C <- Aint + 1L
+    C2 <- 1L + Aint
+    
+    fgpuA <- gpuMatrix(Aint, type="integer")
+    
+    fgpuC <- fgpuA + 1L
+    fgpuC2 <- 1L + fgpuA
+    
+    expect_is(fgpuC, "igpuMatrix")
+    expect_equal(fgpuC[,], C,
+                 info="integer matrix elements not equivalent") 
+    expect_is(fgpuC2, "igpuMatrix")
+    expect_equal(fgpuC2[,], C2,
+                 info="integer matrix elements not equivalent") 
+})
+
+test_that("gpuMatrix Integer Precision Matrix Element-Wise Multiplication", {
+    
+    has_gpu_skip()
+    
+    C <- Aint * Bint
+    
+    fgpuA <- gpuMatrix(Aint, type="integer")
+    fgpuB <- gpuMatrix(Bint, type="integer")
+    
+    fgpuC <- fgpuA * fgpuB
+    
+    expect_is(fgpuC, "igpuMatrix")
+    expect_equal(fgpuC[,], C, 
                  info="integer matrix elements not equivalent")  
 })
 
+test_that("gpuMatrix Integer Precision Scalar Matrix Multiplication", {
+    
+    has_gpu_skip()
+    
+    C <- Aint * 2L
+    C2 <- 2L * Aint
+    
+    dgpuA <- gpuMatrix(Aint, type="integer")
+    
+    dgpuC <- dgpuA * 2L
+    dgpuC2 <- 2L * dgpuA
+    
+    expect_is(dgpuC, "igpuMatrix")
+    expect_equal(dgpuC[,], C,
+                 info="integer matrix elements not equivalent") 
+    expect_is(dgpuC2, "igpuMatrix")
+    expect_equal(dgpuC2[,], C2,
+                 info="integer matrix elements not equivalent") 
+})
+
+test_that("gpuMatrix Integer Precision Matrix Element-Wise Division", {
+    
+    has_gpu_skip()
+    
+    C <- Aint / Bint
+    C <- apply(C, 2, as.integer)
+    
+    fgpuA <- gpuMatrix(Aint, type="integer")
+    fgpuB <- gpuMatrix(Bint, type="integer")
+    
+    fgpuC <- fgpuA / fgpuB
+    
+    expect_is(fgpuC, "igpuMatrix")
+    expect_equal(fgpuC[,], C,
+                 info="integer matrix elements not equivalent")  
+})
+
+test_that("gpuMatrix Integer Precision Scalar Matrix Division", {
+    
+    has_gpu_skip()
+    
+    C <- Aint/2L
+    C2 <- 2L/Aint
+    
+    C <- apply(C, 2, as.integer)
+    C2 <- apply(C2, 2, as.integer)
+    
+    dgpuA <- gpuMatrix(Aint, type="integer")
+    
+    dgpuC <- dgpuA/2L
+    dgpuC2 <- 2L/dgpuA
+    
+    expect_is(dgpuC, "igpuMatrix")
+    expect_equal(dgpuC[,], C,
+                 info="integer matrix elements not equivalent") 
+    expect_is(dgpuC2, "igpuMatrix")
+    expect_equal(dgpuC2[,], C2,
+                 info="integer matrix elements not equivalent") 
+})
+
+test_that("gpuMatrix Integer Precision Matrix Element-Wise Power", {
+
+    has_gpu_skip()
+
+    Apow <- matrix(seq.int(9), ncol=3, nrow=3)
+    Bpow <- matrix(2, ncol = 3, nrow = 3)
+    C <- Apow ^ Bpow
+
+    fgpuA <- gpuMatrix(Apow, type="integer")
+    fgpuB <- gpuMatrix(Bpow, type="integer")
+
+    fgpuC <- fgpuA ^ fgpuB
+
+    expect_is(fgpuC, "igpuMatrix")
+    expect_equal(fgpuC[,], C,
+                 info="integer matrix elements not equivalent")
+})
+
+test_that("gpuMatrix Integer Precision Scalar Matrix Power", {
+
+    has_gpu_skip()
+
+    C <- Aint^2L
+    C2 <- 2L^Aint
+
+    C <- apply(C, 2, as.integer)
+    C2 <- apply(C2, 2, as.integer)
+
+    dgpuA <- gpuMatrix(Aint, type="integer")
+
+    dgpuC <- dgpuA^2L
+    dgpuC2 <- 2L^dgpuA
+
+    expect_is(dgpuC, "igpuMatrix")
+    expect_equal(dgpuC[,], C,
+                 info="integer matrix elements not equivalent")
+    expect_equal(dgpuC2[,], C2,
+                 info="integer matrix elements not equivalent")
+})
+
+# test_that("gpuMatrix Integer Precision crossprod", {
+#     
+#     has_gpu_skip()
+#     
+#     X <- matrix(seq.int(10), nrow=2)
+#     Y <- matrix(seq.int(10), nrow=2)
+#     
+#     C <- crossprod(X,Y)
+#     Cs <- crossprod(X)
+#     
+#     fgpuX <- gpuMatrix(X, type="integer")
+#     fgpuY <- gpuMatrix(Y, type="integer")
+#     
+#     fgpuC <- crossprod(fgpuX, fgpuY)
+#     fgpuCs <- crossprod(fgpuX)
+#     
+#     expect_is(fgpuC, "igpuMatrix")
+#     expect_equal(fgpuC[,], C,
+#                  info="integer matrix elements not equivalent")  
+#     expect_equal(fgpuCs[,], Cs,
+#                  info="integer matrix elements not equivalent") 
+# })
+# 
+# test_that("gpuMatrix Integer Precision tcrossprod", {
+#     
+#     has_gpu_skip()
+#     
+#     X <- matrix(seq.int(10), nrow=2)
+#     Y <- matrix(seq.int(10), nrow=2)
+#     
+#     C <- tcrossprod(X,Y)
+#     Cs <- tcrossprod(X)
+#     
+#     fgpuX <- gpuMatrix(X, type="integer")
+#     fgpuY <- gpuMatrix(Y, type="integer")
+#     fgpuZ <- gpuMatrix(Z, type="integer")
+#     
+#     fgpuC <- tcrossprod(fgpuX, fgpuY)
+#     fgpuCs <- tcrossprod(fgpuX)
+#     
+#     expect_is(fgpuC, "igpuMatrix")
+#     expect_equal(fgpuC[,], C,
+#                  info="integer matrix elements not equivalent")  
+#     expect_equal(fgpuCs[,], Cs,
+#                  info="integer matrix elements not equivalent") 
+# })
+
+# test_that("gpuMatrix Integer Precision transpose", {
+# 
+#     has_gpu_skip()
+# 
+#     At <- t(Aint)
+# 
+#     fgpuA <- gpuMatrix(Aint, type="integer")
+#     fgpuAt <- t(fgpuA)
+# 
+#     expect_is(fgpuAt, "igpuMatrix")
+#     expect_equal(fgpuAt[,], At,
+#                  info="transposed integer matrix elements not equivalent")
+# })
+
 # Double Precision tests
 
 test_that("gpuMatrix Double Precision Matrix multiplication", {
@@ -581,6 +818,7 @@ test_that("gpuMatrix Double Precision Scalar Matrix Power", {
 test_that("gpuMatrix Double Precision crossprod", {
     
     has_gpu_skip()
+    has_double_skip()
     
     X <- matrix(rnorm(10), nrow=2)
     Y <- matrix(rnorm(10), nrow=2)
@@ -607,6 +845,7 @@ test_that("gpuMatrix Double Precision crossprod", {
 test_that("gpuMatrix Double Precision tcrossprod", {
     
     has_gpu_skip()
+    has_double_skip()
     
     X <- matrix(rnorm(10), nrow=2)
     Y <- matrix(rnorm(10), nrow=2)
@@ -645,3 +884,42 @@ test_that("gpuMatrix Double Precision transpose", {
                  info="transposed double matrix elements not equivalent") 
 })
 
+
+test_that("gpuMatrix Diagonal access", {
+    
+    has_gpu_skip()
+    
+    fgpuA <- gpuMatrix(A, type="float")
+    
+    D <- diag(A)
+    gpuD <- diag(fgpuA)
+    
+    expect_is(gpuD, "fgpuVector")
+    expect_equal(gpuD[,], D, tolerance=1e-07, 
+                 info="float matrix diagonal elements not equivalent")  
+    
+    vec <- rnorm(ORDER)
+    diag(fgpuA) <- gpuVector(vec, type = "float")
+    diag(A) <- vec
+    
+    expect_equal(fgpuA[,], A, tolerance=1e-07, 
+                 info="set float matrix diagonal elements not equivalent") 
+    
+    has_double_skip()
+    
+    fgpuA <- gpuMatrix(A, type="double")
+    
+    D <- diag(A)
+    gpuD <- diag(fgpuA)
+    
+    expect_is(gpuD, "dgpuVector")
+    expect_equal(gpuD[,], D, tolerance=.Machine$double.eps^0.5, 
+                 info="double matrix diagonal elements not equivalent")  
+    
+    vec <- rnorm(ORDER)
+    diag(fgpuA) <- gpuVector(vec, type = "double")
+    diag(A) <- vec
+    
+    expect_equal(fgpuA[,], A, tolerance=.Machine$double.eps^0.5, 
+                 info="set double matrix diagonal elements not equivalent") 
+})
