@@ -23,12 +23,15 @@ test_that("vclMatrix Single Precision Matrix Multiplication", {
     
     fvclA <- vclMatrix(A, type="float")
     fvclB <- vclMatrix(B, type="float")
+    fvclE <- vclMatrix(E, type = "float")
     
     fvclC <- fvclA %*% fvclB
     
     expect_is(fvclC, "fvclMatrix")
     expect_equal(fvclC[,], C, tolerance=1e-07, 
                  info="float matrix elements not equivalent")  
+    expect_error(fvclA %*% fvclE, 
+                 info = "error not thrown for non-conformant matrices")
 })
 
 test_that("vclMatrix Single Precision Matrix Subtraction", {
@@ -53,7 +56,6 @@ test_that("vclMatrix Single Precision Scalar Matrix Subtraction", {
     
     C <- A - 1
     C2 <- 1 - A
-    
     fvclA <- vclMatrix(A, type="float")
     
     fvclC <- fvclA - 1    
@@ -72,7 +74,6 @@ test_that("vclMatrix Single Precision Unary Scalar Matrix Subtraction", {
     has_gpu_skip()
     
     C <- -A
-    
     fvclA <- vclMatrix(A, type="float")
     
     fvclC <- -fvclA
@@ -87,7 +88,6 @@ test_that("vclMatrix Single Precision Matrix Addition", {
     has_gpu_skip()
     
     C <- A + B
-    
     fvclA <- vclMatrix(A, type="float")
     fvclB <- vclMatrix(B, type="float")
     
@@ -293,6 +293,226 @@ test_that("vclMatrix Single Precision transpose", {
                  info="transposed float matrix elements not equivalent") 
 })
 
+# Integer tests
+
+test_that("vclMatrix Integer Matrix multiplication", {
+    
+    has_gpu_skip()
+    
+    Cint <- Aint %*% Bint
+    
+    igpuA <- vclMatrix(Aint, type="integer")
+    igpuB <- vclMatrix(Bint, type="integer")
+    
+    igpuC <- igpuA %*% igpuB
+    
+    expect_equivalent(igpuC[,], Cint,
+                      info="integer matrix elements not equivalent")
+})
+
+test_that("vclMatrix Integer Matrix Subtraction", {
+    
+    has_gpu_skip()
+    
+    Cint <- Aint - Bint
+    
+    igpuA <- vclMatrix(Aint, type="integer")
+    igpuB <- vclMatrix(Bint, type="integer")
+    
+    igpuC <- igpuA - igpuB
+    
+    expect_is(igpuC, "ivclMatrix")
+    expect_equal(igpuC[,], Cint,
+                 info="integer matrix elements not equivalent")
+})
+
+test_that("vclMatrix Integer Precision Scalar Matrix Subtraction", {
+    
+    has_gpu_skip()
+    
+    C <- Aint - 1L
+    C2 <- 1L - Aint
+    
+    fgpuA <- vclMatrix(Aint, type="integer")
+    
+    fgpuC <- fgpuA - 1L
+    fgpuC2 <- 1L - fgpuA
+    
+    expect_is(fgpuC, "ivclMatrix")
+    expect_equal(fgpuC[,], C, 
+                 info="integer matrix elements not equivalent") 
+    expect_is(fgpuC2, "ivclMatrix")
+    expect_equal(fgpuC2[,], C2,
+                 info="intger matrix elements not equivalent") 
+})
+
+test_that("vclMatrix Integer Precision Unary Scalar Matrix Subtraction", {
+    
+    has_gpu_skip()
+    
+    C <- -Aint
+    
+    fgpuA <- vclMatrix(Aint, type="integer")
+    
+    fgpuC <- -fgpuA
+    
+    expect_is(fgpuC, "ivclMatrix")
+    expect_equal(fgpuC[,], C,
+                 info="integer matrix elements not equivalent") 
+})
+
+test_that("vclMatrix Integer Matrix Addition", {
+    
+    has_gpu_skip()
+    
+    Cint <- Aint + Bint
+    
+    igpuA <- vclMatrix(Aint, type="integer")
+    igpuB <- vclMatrix(Bint, type="integer")
+    
+    igpuC <- igpuA + igpuB
+    
+    expect_is(igpuC, "ivclMatrix")
+    expect_equal(igpuC[,], Cint,
+                 info="integer matrix elements not equivalent")
+})
+
+test_that("vclMatrix Integer Precision Scalar Matrix Addition", {
+    
+    has_gpu_skip()
+    
+    C <- Aint + 1L
+    C2 <- 1L + Aint
+    
+    fgpuA <- vclMatrix(Aint, type="integer")
+    
+    fgpuC <- fgpuA + 1L
+    fgpuC2 <- 1L + fgpuA
+    
+    expect_is(fgpuC, "ivclMatrix")
+    expect_equal(fgpuC[,], C,
+                 info="integer matrix elements not equivalent") 
+    expect_is(fgpuC2, "ivclMatrix")
+    expect_equal(fgpuC2[,], C2,
+                 info="integer matrix elements not equivalent") 
+})
+
+test_that("vclMatrix Integer Precision Matrix Element-Wise Multiplication", {
+    
+    has_gpu_skip()
+    
+    C <- Aint * Bint
+    
+    fgpuA <- vclMatrix(Aint, type="integer")
+    fgpuB <- vclMatrix(Bint, type="integer")
+    
+    fgpuC <- fgpuA * fgpuB
+    
+    expect_is(fgpuC, "ivclMatrix")
+    expect_equal(fgpuC[,], C, 
+                 info="integer matrix elements not equivalent")  
+})
+
+test_that("vclMatrix Integer Precision Scalar Matrix Multiplication", {
+    
+    has_gpu_skip()
+    
+    C <- Aint * 2L
+    C2 <- 2L * Aint
+    
+    dgpuA <- vclMatrix(Aint, type="integer")
+    
+    dgpuC <- dgpuA * 2L
+    dgpuC2 <- 2L * dgpuA
+    
+    expect_is(dgpuC, "ivclMatrix")
+    expect_equal(dgpuC[,], C,
+                 info="integer matrix elements not equivalent") 
+    expect_is(dgpuC2, "ivclMatrix")
+    expect_equal(dgpuC2[,], C2,
+                 info="integer matrix elements not equivalent") 
+})
+
+test_that("vclMatrix Integer Precision Matrix Element-Wise Division", {
+    
+    has_gpu_skip()
+    
+    C <- Aint / Bint
+    C <- apply(C, 2, as.integer)
+    
+    fgpuA <- vclMatrix(Aint, type="integer")
+    fgpuB <- vclMatrix(Bint, type="integer")
+    
+    fgpuC <- fgpuA / fgpuB
+    
+    expect_is(fgpuC, "ivclMatrix")
+    expect_equal(fgpuC[,], C,
+                 info="integer matrix elements not equivalent")  
+})
+
+test_that("vclMatrix Integer Precision Scalar Matrix Division", {
+    
+    has_gpu_skip()
+    
+    C <- Aint/2L
+    C2 <- 2L/Aint
+    
+    C <- apply(C, 2, as.integer)
+    C2 <- apply(C2, 2, as.integer)
+    
+    dgpuA <- vclMatrix(Aint, type="integer")
+    
+    dgpuC <- dgpuA/2L
+    dgpuC2 <- 2L/dgpuA
+    
+    expect_is(dgpuC, "ivclMatrix")
+    expect_equal(dgpuC[,], C,
+                 info="integer matrix elements not equivalent") 
+    expect_is(dgpuC2, "ivclMatrix")
+    expect_equal(dgpuC2[,], C2,
+                 info="integer matrix elements not equivalent") 
+})
+
+test_that("vclMatrix Integer Precision Matrix Element-Wise Power", {
+    
+    has_gpu_skip()
+    
+    Apow <- matrix(seq.int(9), ncol=3, nrow=3)
+    Bpow <- matrix(2, ncol = 3, nrow = 3)
+    C <- Apow ^ Bpow
+    
+    fgpuA <- vclMatrix(Apow, type="integer")
+    fgpuB <- vclMatrix(Bpow, type="integer")
+    
+    fgpuC <- fgpuA ^ fgpuB
+    
+    expect_is(fgpuC, "ivclMatrix")
+    expect_equal(fgpuC[,], C,
+                 info="integer matrix elements not equivalent")
+})
+
+test_that("vclMatrix Integer Precision Scalar Matrix Power", {
+    
+    has_gpu_skip()
+    
+    C <- Aint^2L
+    C2 <- 2L^Aint
+    
+    C <- apply(C, 2, as.integer)
+    C2 <- apply(C2, 2, as.integer)
+    
+    dgpuA <- vclMatrix(Aint, type="integer")
+    
+    dgpuC <- dgpuA^2L
+    dgpuC2 <- 2L^dgpuA
+    
+    expect_is(dgpuC, "ivclMatrix")
+    expect_equal(dgpuC[,], C,
+                 info="integer matrix elements not equivalent")
+    expect_equal(dgpuC2[,], C2,
+                 info="integer matrix elements not equivalent")
+})
+
 # Double Precision Tests
 
 test_that("vclMatrix Double Precision Matrix multiplication", {
@@ -522,6 +742,7 @@ test_that("vclMatrix Double Precision Scalar Matrix Power", {
 test_that("vclMatrix Double Precision crossprod", {
     
     has_gpu_skip()
+    has_double_skip()
     
     X <- matrix(rnorm(10), nrow=2)
     Y <- matrix(rnorm(10), nrow=2)
@@ -548,6 +769,7 @@ test_that("vclMatrix Double Precision crossprod", {
 test_that("vclMatrix Double Precision tcrossprod", {
     
     has_gpu_skip()
+    has_double_skip()
     
     X <- matrix(rnorm(10), nrow=2)
     Y <- matrix(rnorm(10), nrow=2)
@@ -633,3 +855,44 @@ test_that("vclMatrix Double Precision transpose", {
 #     expect_equal(igpuC[,], Cint,
 #                  info="integer matrix elements not equivalent")  
 # })
+
+
+
+test_that("CPU vclMatrix Diagonal access", {
+    
+    has_gpu_skip()
+    
+    fgpuA <- vclMatrix(A, type="float")
+    
+    D <- diag(A)
+    gpuD <- diag(fgpuA)
+    
+    expect_is(gpuD, "fvclVector")
+    expect_equal(gpuD[,], D, tolerance=1e-07, 
+                 info="float matrix diagonal elements not equivalent")  
+    
+    vec <- rnorm(ORDER)
+    diag(fgpuA) <- vclVector(vec, type = "float")
+    diag(A) <- vec
+    
+    expect_equal(fgpuA[,], A, tolerance=1e-07, 
+                 info="set float matrix diagonal elements not equivalent") 
+    
+    has_double_skip()
+    
+    fgpuA <- vclMatrix(A, type="double")
+    
+    D <- diag(A)
+    gpuD <- diag(fgpuA)
+    
+    expect_is(gpuD, "dvclVector")
+    expect_equal(gpuD[,], D, tolerance=.Machine$double.eps^0.5, 
+                 info="double matrix diagonal elements not equivalent")  
+    
+    vec <- rnorm(ORDER)
+    diag(fgpuA) <- vclVector(vec, type = "double")
+    diag(A) <- vec
+    
+    expect_equal(fgpuA[,], A, tolerance=.Machine$double.eps^0.5, 
+                 info="set double matrix diagonal elements not equivalent") 
+})
